package com.engine.organization.mapper.scheme;


import com.engine.organization.entity.TreeData;
import com.engine.organization.entity.scheme.po.LevelPO;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * @Author dxfeng
 * @description:
 * @Date 2022/5/9
 * @Version V1.0
 **/
public interface LevelMapper {

    /**
     * 根据No查询登记方案
     *
     * @param levelNo
     * @return
     */
    List<LevelPO> listByNo(@Param("levelNo") String levelNo);

    /**
     * 查询所有被引用的ID
     *
     * @return
     */
    List<String> listUsedId();

    /**
     * 获取职等根据ID
     *
     * @param id
     * @return
     */
    LevelPO getLevelByID(@Param("id") long id);

    /**
     * 根据ID查询登记方案列表
     *
     * @param ids
     * @return
     */
    @MapKey("id")
    List<Map<String, Object>> listLevelsByIds(@Param("ids") Collection<Long> ids);

    /**
     * 插入职等
     *
     * @param levelPO
     * @return
     */
    int insertIgnoreNull(LevelPO levelPO);

    /**
     * 修改，修改所有字段
     *
     * @param levelPO
     * @return
     */
    int updateLevel(LevelPO levelPO);

    /**
     * 更新禁用标记
     *
     * @param levelPO
     * @return
     */
    int updateForbiddenTagById(LevelPO levelPO);

    /**
     * 批量删除登记方案
     *
     * @param ids
     */
    int deleteByIds(@Param("ids") Collection<Long> ids);

    /**
     * 根据禁用标记统计
     *
     * @param tag
     * @return
     */
    int getCountByTag(@Param("tag") int tag);

    /**
     * 获取左侧树
     *
     * @return
     */
    ArrayList<TreeData> getTreeData();
}
